@@ -1,6 +1,6 @@
 /*
     GNUbik -- A 3 dimensional magic cube game.
-    Copyright (C) 2003  John Darrington
+    Copyright (C) 2003,2004  John Darrington
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
-static const char RCSID[]="$Id: glarea-gtk.c,v 1.9 2004/01/21 11:25:58 jmd Exp $";
+static const char RCSID[]="$Id: glarea-gtk.c,v 1.10 2004/07/31 09:45:22 jmd Exp $";
 
 
 #include <time.h>
@@ -143,20 +143,27 @@ create_gl_area(GtkWidget *containerWidget, GtkWidget *adjacentWidget)
       { "application/x-color", 0, RDRAG_COLOUR },
   };
 
+  int attribs[]={GDK_GL_RGBA,
+		 GDK_GL_RED_SIZE,   1,
+		 GDK_GL_GREEN_SIZE, 1,
+		 GDK_GL_BLUE_SIZE,  1,
 
+		 GDK_GL_DOUBLEBUFFER,
 
-  int attribs[] = { GDK_GL_RGBA, 
-		    GDK_GL_DEPTH_SIZE, 12, 
-		    GDK_GL_DOUBLEBUFFER,
-		    GDK_GL_RED_SIZE, 1, 
-		    GDK_GL_ACCUM_RED_SIZE, 1,
-		    GDK_GL_ACCUM_GREEN_SIZE, 1,
-		    GDK_GL_ACCUM_BLUE_SIZE, 1,
-		    GDK_GL_ACCUM_ALPHA_SIZE, 1,
-		    GDK_GL_ATTRIB_LIST_NONE };
+		 GDK_GL_DEPTH_SIZE ,1,
+
+		 GDK_GL_ACCUM_RED_SIZE, 1,
+		 GDK_GL_ACCUM_GREEN_SIZE, 1,
+		 GDK_GL_ACCUM_BLUE_SIZE, 1,
+
+		 GDK_GL_ATTRIB_LIST_NONE 
+  };
 
   glconfig = gdk_gl_config_new(attribs);
- 
+
+  if ( !glconfig ) 
+    g_error("Cannot get a suitable visual");
+
   glxarea = gtk_drawing_area_new();
 
   gtk_widget_set_gl_capability (glxarea,
