@@ -1,6 +1,6 @@
 /*
     GNUbik -- A 3 dimensional magic cube game.
-    Copyright (C) 1998, 2003  John Darrington
+    Copyright (C) 1998, 2003, 2004  John Darrington
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 #include "../config.h"
 #include <libintl.h>
 #define _(String) gettext(String)
+#define N_(String) (String)
 
 #ifdef WIDGET_SET_gtk
 #include <gtk/gtk.h>
@@ -46,13 +47,6 @@ extern int cube_dimension;
 extern int number_of_blocks;
 
 
-/* A structure containing information about a movement taking place */
-struct move_data {
-  int slice;
-  int dir; 
-  int axis;
-};
-
 
 #ifndef M_PI
 # define M_PI           3.14159265358979323846
