@@ -16,7 +16,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
-static const char RCSID[]="$Id: main.c,v 1.10 2004/01/06 16:47:06 jmd Exp $";
+static const char RCSID[]="$Id: main.c,v 1.11 2004/02/18 10:56:24 dale_mellor Exp $";
 
 
 #include <time.h>
@@ -69,6 +69,7 @@ c_main(void *closure, int argc, char *argv[])
 {
   t_widget form ;
   t_widget menubar;
+  t_widget play_toolbar;
   t_widget statusbar;
 
 
@@ -100,10 +101,12 @@ c_main(void *closure, int argc, char *argv[])
 
 
   menubar = create_menubar(form,main_application_window);
+  play_toolbar = create_play_toolbar (form, main_application_window);
+  
 
 
 
-  glxarea = create_gl_area(form,menubar);
+  glxarea = create_gl_area(form,play_toolbar);
 
 
   statusbar  = create_statusbar(form);
