@@ -16,7 +16,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
-static const char RCSID[]="$Id: menus-gtk.c,v 1.14 2004/01/22 08:41:00 jmd Exp $";
+static const char RCSID[]="$Id: menus-gtk.c,v 1.16 2004/08/09 10:54:39 jmd Exp $";
 
 #include <gtk/gtk.h>
 #include "menus-gtk.h"
@@ -52,7 +52,7 @@ value_changed(GtkAdjustment *adj,gpointer user_data);
 
 
 #if DEBUG
-static void manual_move(GtkWidget *w , struct move_data * data);
+static void manual_move(GtkWidget *w , Move_Data * data);
 
 static void        
 fill_value(GtkAdjustment *adj, int *p)
@@ -77,14 +77,36 @@ set_lighting(GtkToggleButton *b , gpointer user_data)
 
 
 
-extern int move_count;
+extern Move_Queue *move_queue;
 
-void 
-new_game( GtkWidget *w,
-	       gpointer   data )
+gboolean new_game(gpointer p);
+
+gboolean 
+new_game(gpointer p)
 {
   int i;
-  
+
+  int reallocate = GPOINTER_TO_INT(p);
+
+  if ( is_animating() ) 
+    return TRUE;
+
+  if ( reallocate ) 
+    {
+
+      re_initialize_glarea();
+
+
+      destroy_the_cube ();
+      /* create the cube */
+      number_of_blocks = create_the_cube (cube_dimension);
+      if ( 0 > number_of_blocks ) {
+	print_cube_error (the_cube, "Error creating cube");
+	exit(-1);
+      }
+
+    }
+
   for( i = 0; i < 8*cube_dimension; i++ )
   {
       Slice_Blocks *const slice =
@@ -97,9 +119,30 @@ new_game( GtkWidget *w,
   
   postRedisplay();
 
-  move_count=0;
+  if (move_queue != NULL) free_move_queue (move_queue);
+  move_queue = new_move_queue ();
+  
+  set_toolbar_state (0);
   update_statusbar();
 
+
+  return FALSE;
+}
+
+
+
+/* Request that the game be restarted
+   If data is non zero, then all it's data will be reallocated
+*/
+void 
+request_new_game( GtkWidget *w, gpointer   reallocate )
+{
+
+  request_stop();
+  if (is_animating() ) 
+    abort_animation();
+  gtk_idle_add(new_game,reallocate);
+
 }
 
 
@@ -326,18 +369,7 @@ yesnoresp(GtkWidget *w, gint resp, gpointer data)
 
     cube_dimension = new_dim ;
 
-    re_initialize_glarea();
-
-
-    destroy_the_cube ();
-    /* create the cube */
-    number_of_blocks = create_the_cube (cube_dimension);
-    if ( 0 > number_of_blocks ) {
-      print_cube_error (the_cube, "Error creating cube");
-      exit(-1);
-    }
-
-    new_game(w,data);
+    request_new_game(w,GINT_TO_POINTER(1));
 
     break ;
 
@@ -369,7 +401,7 @@ confirm_preferences(GtkWidget *w , gpointer data)
     /* Popup dialog asking whether to restart the cube */
     GtkWidget *dialog ;
    
-    dialog = gtk_message_dialog_new( main_application_window,
+    dialog = gtk_message_dialog_new( GTK_WINDOW(main_application_window),
 				     GTK_DIALOG_MODAL 
 				     | GTK_DIALOG_DESTROY_WITH_PARENT,
 				     GTK_MESSAGE_QUESTION,
@@ -431,7 +463,7 @@ move( GtkWidget *w,      gpointer   data )
   GtkWidget *label1, *label2,   *label4;
   GtkWidget *sb1,    *sb2,      *sb4;
 
-  static struct move_data md;
+  static Move_Data md;
 
   dialog = gtk_dialog_new();
    
@@ -441,12 +473,12 @@ move( GtkWidget *w,      gpointer   data )
 
 
   hbox1 = gtk_hbox_new(TRUE,0);
-  adj1 = gtk_adjustment_new((cube_dimension+1)%2,-cube_dimension+1,cube_dimension-1,2,1,2);
+  adj1 = gtk_adjustment_new(0,0,number_of_blocks-1,1,1,10);
   
   g_signal_connect(adj1,"value-changed",GTK_SIGNAL_FUNC(fill_value),&md.slice);
 
   sb1 = gtk_spin_button_new(GTK_ADJUSTMENT(adj1),0,0);
-  label1  = gtk_label_new("Slice to move:");
+  label1  = gtk_label_new("Block to move:");
 
 
   hbox2 = gtk_hbox_new(TRUE,0);
@@ -516,7 +548,7 @@ move( GtkWidget *w,      gpointer   data )
 
 /* Manually make a move on the cube */
 static void
-manual_move(GtkWidget *w , struct move_data * data)
+manual_move(GtkWidget *w , Move_Data * data)
 {
 
   /* Just loose the first argument and call the next function */
