@@ -22,7 +22,7 @@
 
 #include <gtk/gtk.h>
 
-void new_game( GtkWidget *w,   gpointer   data );
+void request_new_game( GtkWidget *w,   gpointer   data );
 			       
 void preferences( GtkWidget *w,gpointer   data );
 
