@@ -1,7 +1,7 @@
-
 /*
     User Interface functions for the Cube
     Copyright (C) 1998, 2003  John Darrington
+                  2004  John Darrington, Dale Mellor
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
-static const char RCSID[]="$Id: ui.c,v 1.6 2004/01/05 13:36:17 dale_mellor Exp $";
+static const char RCSID[]="$Id: ui.c,v 1.10 2004/08/16 11:41:17 jmd Exp $";
 
 #include <memory.h>
 #include <stdlib.h>
@@ -42,6 +42,7 @@ static const char RCSID[]="$Id: ui.c,v 1.6 2004/01/05 13:36:17 dale_mellor Exp $
 #endif
 
 
+static short abort_requested=0;
 static Slice_Blocks *blocks_in_motion;
 
 
@@ -59,82 +60,39 @@ int frameQty=2;
 
 static int inverted_rotation  = 0;
 
+
+Move_Queue *move_queue = 0;
+
+
 /* The move taking place NOW */
-struct move_data current_movement;
-
-/*
- * DM 2-JAN-2004
- *   
- *  This used to be 100 but my Guile cube-solver kept hitting the limit, hence
- *  the larger size. Maybe one day it will be more dynamic...
- */
-#define QSIZE 1024
-
-static struct move_data movement_queue[QSIZE];
-static int in_ptr=0;
-static int out_ptr=0;
-
-
-int move_count = 0;
-
-void animate_rotation( struct move_data *data);
-
-
-int queue(struct move_data *m);
-int dequeue(struct move_data *m);
-
-int queue_is_empty(void);
-
-int 
-queue_is_empty(void)
+struct _Current_Movement
 {
-  return (in_ptr == out_ptr);
+    Move_Data move_data;
+    int stop_requested;
+}
+current_movement;
+
+
+/* Toolbar callback. The stop will happen when the current animation
+   completes. */
+void request_stop ()
+{
+    current_movement.stop_requested = 1;
 }
 
-int
-queue(struct move_data *m)
-{
-
-  if ( out_ptr -1  == in_ptr || 
-       ( out_ptr == QSIZE && in_ptr == 0 ) ) /* queue full */ { 
-    return -1;
-  }
-
-  memcpy(movement_queue+in_ptr,m,sizeof(struct move_data));
-
-  in_ptr++; 
-
-  if ( in_ptr >= QSIZE) 
-    in_ptr = 0 ;
 
 
-  return 0;
 
-}
 
-int
-dequeue(struct move_data *m)
-{
+void animate_rotation (Move_Data *data);
 
-  if (in_ptr == out_ptr) /* queue empty */
-    return -1; 
-
-  memcpy(m, movement_queue+out_ptr,sizeof(struct move_data));
-  
-
-  out_ptr++; 
-  if ( out_ptr >= QSIZE )
-    out_ptr = 0;
-
-  return 0;
-}
 
 
 
 
 
 /* The move that will take place when the mouse is clicked */
-struct move_data pending_movement={-1,-1,-1};
+Move_Data pending_movement={-1,-1,-1};
 
 
 
@@ -147,12 +105,19 @@ double radians(int deg);
 extern Quarternion cube_view;
 
 
+int
+is_animating(void)
+{
+  return animation_in_progress;
+}
+
+
 /* Rotate cube about axis (screen relative) */
 void
 rotate_cube(int axis, int dir)
 {
   /* how many degrees to turn the cube with each hit */
-  GLdouble step=5.0;
+  GLdouble step=2.0;
   Quarternion rot;
   vector v;
 
@@ -231,13 +196,7 @@ turnDir(GLfloat *vector)
   /* This is a horrendous kludge.  It works only because we know that 
      vector is arithemtically very simple */
 
-  int i;
-  int x=0;
-  for (i=0;i<3;i++ ){
-    x+=vector[i];
-  }
-  return ( x > 0 );
-	
+    return (vector [0] + vector [1] + vector [2]  >  0);
 }
 
 
@@ -248,18 +207,7 @@ is orthogonal to the frame of reference ( another nasty kludge ). */
 int
 vector2axis(GLfloat *vector)
 {
-  int i, axis=-1;
-  for (i=0;i<3;i++ ){
-    if(vector[i] != 0) {
-      axis = i;		
-      break;
-    }
-  }
-  if ( axis == -1 ) {
-    fprintf(stderr,"Funny vector\n");
-    exit(1);
-  }
-  return axis;
+    return vector [0] != 0 ? 0 : vector [1] != 0 ? 1 : 2;
 }
 	
 
@@ -397,10 +345,10 @@ drawCube(void)
 
       {
 	int unity=1;
-	if ( ! current_movement.dir) 
+	if ( ! current_movement.move_data.dir) 
 	  unity = -1;
 
-	switch (current_movement.axis) { 
+	switch (current_movement.move_data.axis) { 
 	case 0: case 3:
 	  glRotatef(angle,unity,0,0);
 	  break ;
@@ -489,58 +437,250 @@ mouse(int button)
 }
 
 
+
+/* Toolbar callback. Use stop_requested to ensure that only one step occurs. */
+
 void
-request_rotation(struct move_data *data)
+request_back ()
 {
+    if (! animation_in_progress  &&  move_queue  &&
+                                               move_queue_retard (move_queue))
+    {
+        current_movement.stop_requested = 1;
+        
+        Move_Data next_move;
 
-  queue(data);
-
-  if ( ! animation_in_progress ) { 
-    struct move_data next_move;
+        memcpy (&next_move,
+                move_queue_current (move_queue),
+                sizeof (Move_Data));
 
+        next_move.dir = ! next_move.dir;
+        
+        animate_rotation (&next_move);
+    }
+}
     
-    dequeue(&next_move);
-    animate_rotation(&next_move);
-  }
 
+
+/* Internal functionality for the following two functions. */
+
+static void
+_request_play (const int one_move)
+{
+    if (! animation_in_progress  &&  move_queue  &&
+                                               move_queue_current (move_queue))
+    {
+        current_movement.stop_requested = one_move;
+        
+        Move_Data next_move;
+
+        memcpy (&next_move,
+                move_queue_current (move_queue),
+                sizeof (next_move));
+        
+        move_queue_advance (move_queue);
+        
+        animate_rotation (&next_move);
+    }
+}
+    
+
+/* Toolbar `play' callback. */
+void
+request_play () 
+{
+    _request_play (0);
+}
+
+/* Toolbar `forward' callback. */
+void
+request_forward ()
+{
+    _request_play (1);
+}
+
+
+
+/* The usual rotation request, called when user uses mouse to rotate the cube by
+   hand. The move is put into the `current' place, and all the moves in the
+   move_queue afterwards are zapped. */
+
+void
+request_rotation (Move_Data *const data)
+{
+    if (move_queue == NULL)
+        move_queue = new_move_queue ();
+    
+    move_queue_push_current (move_queue, data);
+
+    request_play ();
+}
+
+
+
+
+/* The rotation request called from script-fu. In this case the moves are
+   appended to the move_queue, so the script can race ahead and work out the
+   moves, leaving the graphics to catch up later. */
+
+void
+request_delayed_rotation (Move_Data *const data)
+{
+    if (move_queue == NULL)
+        move_queue = new_move_queue ();
+    
+    move_queue_push (move_queue, data);
+}
+
+
+
+
+/* Script-fu calls this function on startup, and then the above function as each
+   move is computed. This way, the moves computed are appended to the move_queue
+   from the current point forwards (and forward data that was on the queue is
+   discarded). */
+
+void
+request_truncate_move_queue ()
+{
+    if (move_queue)
+        move_queue_truncate (move_queue);
+}
+
+
+
+/* Toolbar callback. */
+
+void
+request_mark_move_queue ()
+{
+    if (move_queue)
+        move_queue_mark_current (move_queue);
+}
+
+
+
+
+/* Toolbar callback. All the moves take place on the_cube, not via the animation
+   routines, so the effect is to unwind the queue instantaneously. */
+
+void
+request_queue_rewind ()
+{
+    int mark_result;
+    Slice_Blocks *blocks;
+    const Move_Data *move_data;
+
+    if (! move_queue)
+        return;
+    
+    do
+    {
+        if ((mark_result = move_queue_retard (move_queue)) == 0)
+            break;
+        
+        if ((move_data = move_queue_current (move_queue)) != NULL)
+        {
+            blocks = identify_blocks_2 (the_cube,
+                                        move_data->slice,
+                                        move_data->axis);
+            
+            rotate_slice (the_cube,
+                          move_data->dir == 0 ? 1 : 3,  /* This is backwards. */
+                          blocks);
+
+            free_slice_blocks (blocks);
+        }
+    }
+    while (mark_result == 1);
+    
+    
+    postRedisplay ();
+    set_toolbar_state ((move_queue_progress (move_queue).current == 0
+                                                      ? 0 : PLAY_TOOLBAR_BACK)
+                       |
+                       (move_queue_current (move_queue)
+                                                      ? PLAY_TOOLBAR_PLAY : 0));
+    
+    update_statusbar ();
+}
+
+
+
+
+/* Script callback. */
+
+void
+request_fast_forward ()
+{
+    const Move_Data *move_data;
+    Slice_Blocks *blocks;
+    
+    if (! move_queue)
+        return;
+
+    for (move_data = move_queue_current (move_queue);
+         move_data != NULL;
+         move_data = (move_queue_advance (move_queue),
+                      move_queue_current (move_queue)))
+    {
+        blocks = identify_blocks_2 (the_cube,
+                                    move_data->slice,
+                                    move_data->axis);
+
+        rotate_slice (the_cube,
+                      move_data->dir == 0 ? 3 : 1,
+                      blocks);
+
+        free_slice_blocks (blocks);
+    }
+
+    postRedisplay ();
+    set_toolbar_state (PLAY_TOOLBAR_BACK);
+    update_statusbar ();
+}
+
+
+
+/* Does exactly what it says on the tin :-) */
+
+void
+animate_rotation (Move_Data *data)
+{
+    memcpy (&current_movement.move_data, data, sizeof (Move_Data));
+
+    blocks_in_motion = identify_blocks_2 (the_cube, data->slice, data->axis);
+
+    if (blocks_in_motion == NULL)
+    {
+        print_cube_error (the_cube, "Error rotating block");
+        exit (-1);
+    }
+    
+    animation_in_progress = 1;
+
+    set_toolbar_state (PLAY_TOOLBAR_STOP);
+
+    add_timeout (picture_rate, animate, &current_movement);
 }
 
 
 void
-animate_rotation( struct move_data *data)
+abort_animation(void)
 {
-
-  struct move_data *md = &current_movement;
-
-
-  assert(data);
-
-  memcpy(md,data,sizeof(struct move_data));
-
-  blocks_in_motion = identify_blocks_2 (the_cube, md->slice, md->axis);
-
-  if ( blocks_in_motion == NULL ) {
-    print_cube_error (the_cube, "Error rotating block");
-    exit(-1);
-  }
-  animation_in_progress = 1;
-
-  add_timeout(picture_rate,animate,md);
+  abort_requested=1;
 }
 
 
-
 /* a timeout  calls this func, to animate the cube */
 TIMEOUT_CALLBACK(animate)
 {
 
-  struct move_data *md = (struct move_data *) data;
+  struct _Current_Movement *md = (struct _Current_Movement *) data;
 
   /* how many degrees motion per frame */
   GLfloat increment = 90.0 / (frameQty +1 ) ;
 
-  assert(md);
-
 
   /*  decrement the current angle */
   animation_angle-= increment ;
@@ -550,7 +690,7 @@ TIMEOUT_CALLBACK(animate)
   postRedisplay();
 
 
-  if ( fabs(animation_angle) < 90.0 ) {
+  if ( fabs(animation_angle) < 90.0 && !abort_requested) {
     /* call this timeout again */
     add_timeout(picture_rate,animate,data);
   }
@@ -561,7 +701,7 @@ TIMEOUT_CALLBACK(animate)
 
     animation_angle = 0.0;
     /* -90 degrees equals +270 degrees */
-    switch ( md->dir ) {
+    switch ( md->move_data.dir ) {
     case 1:
       turn_qty=1;
       break;
@@ -582,21 +722,35 @@ TIMEOUT_CALLBACK(animate)
     
     animation_in_progress = 0;
 
-    ++move_count;
-
+    set_toolbar_state ((move_queue_progress (move_queue).current == 0
+                                                      ? 0 : PLAY_TOOLBAR_BACK)
+                       |
+                       (move_queue_current (move_queue)
+                                                      ? PLAY_TOOLBAR_PLAY : 0));
+    
     update_statusbar();
     updateSelection();
 
     if (  NOT_SOLVED != (status =  cube_status_check(the_cube) ) )
       declare_win(status);
 
-    /* If there are more animations in the queue, go again */
-    if ( !queue_is_empty() ) {
-      struct move_data next_move;
-      
-      dequeue(&next_move);
-      
-      animate_rotation(&next_move);
+    if ( abort_requested ) 
+      abort_requested = 0;
+    
+    /* If there are more animations in the queue, go again unless a stop has
+       been requested. */
+    
+    if (md->stop_requested)
+        md->stop_requested = 0;
+
+    
+    else if (move_queue_current (move_queue))
+    {
+      memcpy (&md->move_data,
+              move_queue_current (move_queue),
+              sizeof (Move_Data));
+      move_queue_advance (move_queue);
+      animate_rotation (&md->move_data);
     }
       
 
