@@ -23,6 +23,7 @@
 #include <GL/gl.h>
 
 #include "gnubik.h"
+#include "move-queue.h"
 
 extern int frameQty;
 
@@ -69,6 +70,8 @@ typedef guint t_keysym;
 
 #endif
 
+void abort_animation(void);
+
 void selection_func();
 void arrows (t_keysym keysym, int shifted);
 void mouse(int button);
@@ -76,7 +79,17 @@ void turn_indicator(int axis, int dir);
 int vector2axis(GLfloat *vector);
 void drawCube(void);
 
-void request_rotation( struct move_data *data) ;
+void request_stop (void);
+void request_play (void);
+void request_forward (void);
+void request_back (void);
+void request_truncate_move_queue (void);
+void request_delayed_rotation (Move_Data *data);
+void request_mark_move_queue (void);
+void request_queue_rewind (void);
+void request_fast_forward (void);
+
+void request_rotation( Move_Data *data) ;
 
 /* Rotate the cube about the axis (screen relative) in direction dir */
 void rotate_cube(int axis, int dir) ;
