@@ -1,6 +1,6 @@
 /*
     GNUbik -- A 3 dimensional magic cube game.
-    Copyright (C) 2003  John Darrington
+    Copyright (C) 2003,2004  John Darrington
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,14 +16,15 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
-static const char RCSID[]="$Id: widget-set-gtk.c,v 1.16 2004/01/21 13:37:24 jmd Exp $";
+static const char RCSID[]="$Id: widget-set-gtk.c,v 1.22 2004/08/18 12:14:05 jmd Exp $";
 
 #include "widget-set.h"
 #include "menus-gtk.h"
 #include "colour-sel.h"
 #include "version.h"
 #include "guile-hooks.h"
-
+#include "move-queue.h"
+#include "ui.h"
 
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
@@ -90,7 +91,7 @@ create_container_widget(t_widget parent)
 
 
 static GtkWidget *statusbar;
-extern int move_count;
+extern Move_Queue *move_queue;
 #define MSGLEN 100
 void
 update_statusbar(void)
@@ -106,9 +107,12 @@ update_statusbar(void)
     context  = gtk_statusbar_get_context_id(GTK_STATUSBAR(statusbar),
 					     "move-count");
 
-
-  g_snprintf(mesg,MSGLEN,_("Moves: %d"),move_count);
-
+  g_snprintf(mesg,
+             MSGLEN,
+             _("Moves: %d / %d"),
+             move_queue_progress (move_queue).current,
+             move_queue_progress (move_queue).total);
+  
   if ( mesg_id != 0 ) 
     gtk_statusbar_remove(GTK_STATUSBAR(statusbar),context,mesg_id);
 
@@ -132,8 +136,9 @@ declare_win(Cube_Status status)
   switch (status ) {
   case SOLVED:
     g_snprintf(mesg,MSGLEN, ngettext ("Cube solved in %d move",
-				      "Cube solved in %d moves", move_count),
-	       move_count);
+                                      "Cube solved in %d moves",
+                                      move_queue_progress (move_queue).total),
+               move_queue_progress (move_queue).total);
     break ;
   case HALF_SOLVED:
     g_snprintf(mesg,MSGLEN,_("Cube is NOT solved! The colours are correct, but have incorrect orientation"));
@@ -164,6 +169,133 @@ create_statusbar(t_widget container)
 
 }
 
+
+
+static GList *play_button_list;
+
+void
+set_toolbar_state (unsigned flags)
+{
+    GList *list;
+
+    for (list = play_button_list; list != NULL; list = list->next)
+    {
+        gtk_widget_set_sensitive (GTK_WIDGET (list->data),
+                                  flags & 1);
+        flags >>= 1;
+    }
+}
+
+
+static GtkWidget *play_toolbar;
+
+
+t_widget
+create_play_toolbar (t_widget container, t_widget toplevel)
+{
+
+    play_toolbar = gtk_toolbar_new ();
+    gtk_toolbar_set_style (GTK_TOOLBAR (play_toolbar), GTK_TOOLBAR_BOTH);
+    gtk_container_set_border_width (GTK_CONTAINER (play_toolbar), 0);
+    gtk_box_pack_start(GTK_BOX(container),play_toolbar,FALSE,TRUE,0);
+
+    gtk_toolbar_append_item (GTK_TOOLBAR (play_toolbar),
+                             _("Rewind"),
+                             _("Go to the previous mark (or the beginning) of the sequence of moves"),
+                             "Private",
+                             gtk_image_new_from_file (ICONDIR "/rewind.xpm"),
+                             request_queue_rewind, container);
+
+    gtk_toolbar_append_item (GTK_TOOLBAR (play_toolbar),
+                             _("Back"),
+                             _("Make one step backwards"),
+                             "Private",
+                             gtk_image_new_from_file (ICONDIR "/back.xpm"),
+                             request_back, container);
+
+    gtk_toolbar_append_item (GTK_TOOLBAR (play_toolbar),
+                             _("Stop"),
+                             _("Stop running the sequence of moves"),
+                             "Private",
+                             gtk_image_new_from_file (ICONDIR "/stop.xpm"),
+                             request_stop, container);
+
+    gtk_toolbar_append_item (GTK_TOOLBAR (play_toolbar),
+                             _("Mark"),
+                             _("Mark the current place in the sequence of moves"),
+                             "Private",
+                             gtk_image_new_from_file (ICONDIR "/mark.xpm"),
+                             request_mark_move_queue, container);
+
+    gtk_toolbar_append_item (GTK_TOOLBAR (play_toolbar),
+                             _("Forward"),
+                             _("Make one step forwards"),
+                             "Private",
+                             gtk_image_new_from_file (ICONDIR "/forward.xpm"),
+                             request_forward, container);
+
+    gtk_toolbar_append_item (GTK_TOOLBAR (play_toolbar),
+                             _("Play"),
+                             _("Run forward through the sequence of moves"),
+                             "Private",
+                             gtk_image_new_from_file (ICONDIR "/play.xpm"),
+                             request_play, container);
+
+    gtk_widget_show (play_toolbar);
+
+    play_button_list
+        = gtk_container_get_children (GTK_CONTAINER (play_toolbar));
+
+    set_toolbar_state (0);
+    
+    return play_toolbar;
+}
+
+
+
+/* Toggle the visibility of a widget */
+void
+toggle_visibility(GtkMenuItem *menuitem,
+		  gpointer user_data)
+{
+  GtkWidget **w = (user_data);
+
+  g_assert(*w);
+
+  if ( GTK_WIDGET_VISIBLE(*w))
+    gtk_widget_hide(*w);
+  else
+    gtk_widget_show(*w);
+}
+
+static GtkWidget *
+create_show_hide_menu (void)
+{
+  GtkWidget *menu;
+  GtkWidget *menuitem;
+
+  menu = gtk_menu_new ();
+
+  menuitem = gtk_menu_item_new_with_mnemonic (_("_Play Toolbar"));
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+  gtk_widget_show (menuitem);
+
+  g_signal_connect (menuitem, "activate",
+			G_CALLBACK (toggle_visibility), &play_toolbar);
+      
+  menuitem = gtk_menu_item_new_with_mnemonic (_("_Status Bar"));
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+  gtk_widget_show (menuitem);
+
+  g_signal_connect (menuitem, "activate",
+			G_CALLBACK (toggle_visibility), &statusbar);
+
+
+
+  return menu;
+}
+
+
 t_widget
 create_menubar(t_widget container,t_widget toplevel)
 {
@@ -171,53 +303,34 @@ create_menubar(t_widget container,t_widget toplevel)
   int i=0;
 
 
-  static GtkItemFactoryEntry menu_items[] = {
-    { 0,         NULL,         NULL, 0, "<Branch>" },
-    { 0,     "<control>N", new_game, 0, NULL },
-    { 0,     NULL,         NULL, 0, "<Separator>" },
-    { 0,     "<control>Q", gtk_main_quit, 0, NULL },
-    { 0,      NULL,         NULL, 0, "<Branch>" },
-    { 0,  NULL, preferences, 0, NULL },
-    { 0,  NULL, colour_select_menu, 0, 0 },
+  const static  GtkItemFactoryEntry menu_items[] = {
+    { N_("/_Game"),         NULL,         NULL, 0, "<Branch>" },
+    { N_("/_Game/_New Game"),     "<control>N", request_new_game, 0, NULL },
+    { N_("/_Game/sep"),     NULL,         NULL, 0, "<Separator>" },
+    { N_("/_Game/_Quit"),     "<control>Q", gtk_main_quit, 0, 
+      "<StockItem>",GTK_STOCK_QUIT },
+    { N_("/_Settings"),      NULL,         NULL, 0, "<Branch>" },
+    { N_("/_Settings/_Preferences"),  NULL, preferences, 0, 
+      "<StockItem>", GTK_STOCK_PREFERENCES },
+    { N_("/_Settings/_Colours"),  NULL, colour_select_menu, 0, 
+      "<StockItem>", GTK_STOCK_SELECT_COLOR },
+    { N_("/_Settings/Sho_w\\/Hide"),  NULL, 0, 0, NULL },
+
+#ifdef HAVE_GUILE
+    { ("/Script-_fu"),  NULL, NULL, 0, "<Branch>" },
+#endif
 #if DEBUG
     { "/_Debug",        NULL,         NULL, 0, "<Branch>" },
     { "/_Debug/_Move",     NULL,      move, 0, NULL },
 #endif
-    { 0,         NULL,         NULL, 0, "<Branch>" },
-    { 0,   NULL,     about, 0, NULL },
+    { N_("/_Help"),         NULL,         NULL, 0, "<Branch>" },
+    { N_("/_Help/_About"),   NULL,     about, 0, NULL },
   };
 
-
   GtkItemFactory *item_factory;
   GtkAccelGroup *accel_group;
-  gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);
 
-  menu_items[i++].path=g_strdup(_("/_Game"));
-  menu_items[i++].path=g_strdup(_("/_Game/_New Game"));
-  menu_items[i++].path=g_strdup(_("/_Game/sep"));
-  menu_items[i++].path=g_strdup(_("/_Game/_Quit"));
-
-  menu_items[i++].path=g_strdup(_("/_Settings"));
-
-  menu_items[i].path=g_strdup(_("/_Settings/_Preferences"));
-  menu_items[i++].callback_action=(gint)toplevel;  
-
-
-  menu_items[i].path=g_strdup(_("/_Settings/_Colours"));
-  menu_items[i++].callback_action=(gint)toplevel;
-
-#ifdef HAVE_GUILE
-  menu_items[i++].path=g_strdup(/*_*/("/Script-_fu"));
-#endif
-  
-  
-#if DEBUG
-  i+=2;
-#endif
-  menu_items[i++].path=g_strdup(_("/_Help"));
-
-  menu_items[i].path=g_strdup(_("/_Help/_About"));
-  menu_items[i++].callback_action=(gint)toplevel;
+  const gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);
 
   accel_group = gtk_accel_group_new ();
 
@@ -238,15 +351,7 @@ create_menubar(t_widget container,t_widget toplevel)
   gtk_item_factory_create_items (item_factory, nmenu_items, 
 				 (GtkItemFactoryEntry*) menu_items, NULL);
 
-  for(i = 0 ; i < nmenu_items ; ++i ) {
-#if DEBUG
-    if ( i == 7 || i == 8 ) 
-      continue;
-#endif
-    g_free(menu_items[i].path);
-  }
 
-  
 #ifdef HAVE_GUILE
   
  {
@@ -262,9 +367,18 @@ create_menubar(t_widget container,t_widget toplevel)
  }
 
 #endif
-
   
-  /* Attach the new accelerator group to the window. */
+ {
+   GtkWidget *w;
+
+   w = gtk_item_factory_get_widget(item_factory,N_("/Settings/Show\\/Hide"));
+   g_assert(w);
+
+   gtk_menu_item_set_submenu (GTK_MENU_ITEM (w), create_show_hide_menu());
+   
+ }
+
+   /* Attach the new accelerator group to the window. */
   gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
 
 
@@ -280,7 +394,8 @@ create_menubar(t_widget container,t_widget toplevel)
 
 }
 
-void cleanup(void)
+void
+cleanup(void)
 {
 }
 
