@@ -34,6 +34,17 @@ t_widget create_container_widget(t_widget parent);
 t_widget create_menubar(t_widget container,t_widget toplevel);
 
 
+t_widget create_play_toolbar (t_widget container, t_widget toplevel);
+enum
+    {
+        PLAY_TOOLBAR_BACK = 0x0b << 0,
+        PLAY_TOOLBAR_STOP = 0x01 << 2,
+        PLAY_TOOLBAR_PLAY = 0x03 << 4
+    } ;
+void set_toolbar_state (unsigned state);
+
+
+
 t_widget create_statusbar(t_widget container);
 void update_statusbar(void);
 
